# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


"""
mac: NetQ app module for processing MAC table and MAC addresses

Usage:
   netq [<hostname>] show mac-history <mac> [vlan <1-4096>] (history | diff) [past <text-endtime>] [between <text-time> and <text-endtime>] [groupby <text-group-by>] [json]

Options:
   mac-history                  : Mac history info for a mac address
   history                      : All change events for mac
   diff                         : Absolute difference throughout the change events for mac
   past                         : Relative time in s,m,h e.g., 2h, 45m, etc
   groupby                      : Group by specific attribute/s
   <1-4096>                     : VLAN ID, between 1-4096
   <text-group-by>              : Comma separated columns names to group the history
"""

from __future__ import absolute_import

from collections import OrderedDict

from netq_apps.modules import NetqModule
from netq_apps.cmd.netq import netq_show
from netq_apps.lib.netq import get_time

from netq_lib.common.enums import HeaderLength
from netq_lib.orm.redisdb.models import MacFdb

app = NetqModule()

@app.route('show mac-history')
@netq_show
def cli_show_machistory(cli, netq, context):
    '''Handle showing of MAC entries'''

    mac = cli.get('<mac>')
    vlan = cli.get('<1-4096>')

    if vlan is not None:
        vlan = int(vlan)
    else:
        vlan = '*'

    hostname = cli.get('<hostname>') or '*'
    diff_type = cli.get('history') or cli.get('diff')
    group_by = cli.get('<text-group-by>')

    field_mapping = OrderedDict([
        ('Last Changed', 'timestamp'),
        ('Hostname', 'hostname'),
        ('VLAN', 'vlan'),
        ('Origin', 'origin'),
        ('Nexthop', 'nexthop'),
        ('Destination', 'dst'),
        ('Port', 'port'),
        ('Remote', 'is_remote'),
        ('Static', 'is_static')
    ])

    end_time = get_time(cli.get('<text-time>')) if cli.get('<text-time>') else get_time('now')
    start_time = get_time(cli.get('<text-endtime>')) if cli.get('<text-endtime>') else get_time('1h')

    if end_time < start_time:
        temp = end_time
        end_time = start_time
        start_time = temp

    group_by_list = []

    if group_by is not None:

        group_by_list = [col.lower() for col in group_by.split(',')]
        supported_groupby_columns = [x.lower() for x in field_mapping.keys()]

        if not(set(group_by_list) <= set(supported_groupby_columns[1:])):
            print('Unsupported column in group by')
            return get_blank()

    context.col_headers = field_mapping.keys()
    context.col_sizes = [HeaderLength.get_col_size(header) for header in context.col_headers]
    context.sort_fn_str = 'lambda x: lambda y: 1'
    context.sort_cols = [0]
    return get_mac_diff(mac, hostname, vlan, diff_type, start_time, end_time, field_mapping, ','.join(group_by_list) or '*')

def get_blank():
    yield OrderedDict()

def get_mac_diff(mac_address, hostname, vlan, diff_type, start_time, end_time, field_mapping, group_by):

    entries = MacFdb.query.request(mac_address=mac_address, start=start_time, end=end_time, hostname=hostname,
                                   vlan=vlan, diff_type=diff_type, return_only_diff='true', group_by=group_by)

    response = entries.next()
    last_complete_object = {}

    if response.obj:
        main_obj = response.obj
        out_obj, last_complete_object = format_macfdb(main_obj, field_mapping, last_complete_object, group_by)
        yield out_obj

    for obj in response.diff:
        out_obj, last_complete_object = format_macfdb(obj, field_mapping, last_complete_object, group_by)
        yield out_obj

def format_macfdb(obj, field_mapping, last_complete_obj, group_by):
    out = OrderedDict()

    return_complete_obj = True if 'hostname' in obj and group_by == '*' else False

    for output_field, input_field in field_mapping.iteritems():

        if return_complete_obj:
            out[output_field] = obj[input_field] if input_field in obj else last_complete_obj[output_field]
        else:
            out[output_field] = obj[input_field] if input_field in obj else '-'

        #Transforming boolean to yes, no
        if isinstance(out[output_field], bool):
            out[output_field] = 'yes' if(out[output_field]) else 'no'

        if output_field == 'Last Changed':
            out[output_field] = float(obj[input_field]) / 1000.0

    complete_obj = out if return_complete_obj else last_complete_obj
    return out, complete_obj
